#!/bin/bash

set -e

# ensure the job has passed us all the information we need to run
REQUIRED_VARS="WORKSPACE IMAGE_BASE IMAGE_ARCH IMAGE_TYPE IMAGE_DEST INSTALL_TYPE RUNLIST"
for var in $REQUIRED_VARS ; do
	# NOTE: this kind of checking requires bash!
	if [ -z ${!var} ] ; then
		echo "Missing required variable $var"
		FAILED=1
	fi
done
[ -z $FAILED ] || exit 1

cleanup() {
	# utah can create files not readable by jenkins without this
	sudo chmod -R o+r ./
}

# now start the real work
set -x

UTAH_CMD="${UTAH_CMD-/usr/share/utah/examples/run_utah_tests.py}"
PROVISION_RUNLIST="${PROVISION_RUNLIST-/usr/share/utah/client/examples/pass.run}"
MACHINE="upgrade-$IMAGE_BASE-$IMAGE_ARCH-$IMAGE_TYPE"

# Create custom configuration file and separate per-job log files
UTAH_CONFIG_DIR=${WORKSPACE}/utah_config
UTAH_CONFIG_FILE=${UTAH_CONFIG_DIR}/utah.cfg
LOG_DIR=${WORKSPACE}/clientlogs
UTAH_LOG_FILE=${LOG_DIR}/utah-server.log
UTAH_DEBUGLOG_FILE=${LOG_DIR}/utah-server-debug.log
UTAH_SSH_LOG_FILE=${LOG_DIR}/utah-server-ssh.log

VM_COPIES=/var/cache/utah/upgrade-testing
CACHED_COPY=${VM_COPIES}/${MACHINE}.qcow2

UTAH_DIRS="${LOG_DIR} ${UTAH_CONFIG_DIR} ${VM_COPIES}"
UTAH_FILES="$UTAH_LOG_FILE $UTAH_DEBUGLOG_FILE $UTAH_SSH_LOG_FILE"

trap cleanup TERM INT EXIT
umask 002
mkdir -p ${UTAH_DIRS}

touch ${UTAH_FILES}
cat <<EOF > ${UTAH_CONFIG_FILE}
{
    "debuglog": "${UTAH_DEBUGLOG_FILE}",
    "logfile": "${UTAH_LOG_FILE}",
    "ssh_logfile": "${UTAH_SSH_LOG_FILE}",
    "logpath": "${LOG_DIR}",
    "jobtimeout": 86400
}
EOF

sudo chgrp -R utah ${UTAH_DIRS}
sudo chmod -R g+w ${UTAH_DIRS}

if [ ! -f ${CACHED_COPY} ] ; then
 	echo "no cached image found, provisioning new image for this"
	PARAMS="-d -s $IMAGE_BASE -t $INSTALL_TYPE -a $IMAGE_ARCH --name $MACHINE -n -g 80"
	sudo -u utah -i UTAH_CONFIG_DIR=${UTAH_CONFIG_DIR} $UTAH_CMD $PARAMS $PROVISION_RUNLIST

	echo "Caching image as: $CACHED_COPY ..."
	IMG=`virsh domblklist $MACHINE | grep vda | awk '{print $NF}'`
	virsh destroy $MACHINE
	sudo cp $IMG $CACHED_COPY
	sudo chmod g+w $CACHED_COPY
	virsh start $MACHINE
	echo "VM re-started, sleeping for a bit before launching utah..."
	sleep 30
else
	echo "restoring original image ..."
	IMG=`virsh domblklist $MACHINE | grep vda | awk '{print $NF}'`
	if virsh list --all | grep $MACHINE | grep running ; then
		virsh destroy $MACHINE
	fi
	sudo cp $CACHED_COPY $IMG
	sudo chmod g+w $IMG   #make sure all utah users can write this file
	virsh start $MACHINE
	echo "VM started, sleeping for a bit before launching utah..."
	sleep 30
fi

set +e
PARAMS="-d --name $MACHINE --skip-provisioning -n"
sudo -u utah -i UTAH_CONFIG_DIR=${UTAH_CONFIG_DIR} $UTAH_CMD $PARAMS $RUNLIST
RETCODE=$?

SCP="sudo -u utah -i scp -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no"
${SCP} root@${MACHINE}:/var/log/dist-upgrade/* ${LOG_DIR}/
${SCP} root@${MACHINE}:/var/crash/* ${LOG_DIR}/

exit $RETCODE
